# -*- coding: utf-8 -*-
import datetime

import pytest


def test_setup_and_info(store):
    # GIVEN a store which is already setup
    assert len(store.engine.table_names()) > 0
    # THEN it should contain an Info entry with a default "created_at" date
    info_obj = store.info()
    assert isinstance(info_obj.created_at, datetime.datetime)


def test_track_update(store):
    # GIVEN a store which is empty apart from the initialized info entry
    assert store.info().updated_at is None
    # WHEN updating the last updated date
    store.track_update()
    # THEN it should update info entry with the current date
    assert isinstance(store.info().updated_at, datetime.datetime)


def test_add_user(store):
    # GIVEN an empty database
    assert store.User.query.first() is None
    name, email = 'Paul T. Anderson', 'paul.anderson@magnolia.com'
    # WHEN adding a new user
    new_user = store.add_user(name, email)
    # THEN it should be stored in the database
    assert store.User.query.filter_by(email=email).first() == new_user


def test_user(store):
    # GIVEN a database with a user
    name, email = 'Paul T. Anderson', 'paul.anderson@magnolia.com'
    store.add_user(name, email)
    assert store.User.query.filter_by(email=email).first().email == email
    # WHEN querying for a user
    user_obj = store.user(email)
    # THEN it should be returned
    assert user_obj.email == email

    # WHEN querying for a user that doesn't exist
    user_obj = store.user('this_is_a_made_up_email@fake_example.com')
    # THEN it should return as None
    assert user_obj is None


def test_analysis(sample_store):
    # GIVEN a store with an analysis
    existing_analysis = sample_store.analyses().first()
    # WHEN accessing it by ID
    analysis_obj = sample_store.analysis(existing_analysis.id)
    # THEN it should return the same analysis
    assert analysis_obj == existing_analysis

    # GIVEN an id that doesn't exist
    missing_analysis_id = 12312423534
    # WHEN accessing the analysis
    analysis_obj = sample_store.analysis(missing_analysis_id)
    # THEN it should return None
    assert analysis_obj is None


@pytest.mark.parametrize('family, expected_bool', [
    ('crazygoat', True),     # running
    ('nicemouse', False),    # completed
    ('politesnake', False),  # failed
    ('gentlebird', True),    # pending
])
def test_is_running(sample_store, family, expected_bool):
    # GIVEN an analysis
    analysis_objs = sample_store.analyses(family=family).first()
    assert analysis_objs is not None
    # WHEN checking if the family has a running analysis
    is_running = sample_store.is_running(family)
    # THEN it should return the expected result
    assert is_running is expected_bool


def test_aggregate_jobs(sample_store):
    # GIVEN a store with some analyses
    assert sample_store.analyses().count() > 0
    all_jobs = sample_store.jobs().count()
    assert all_jobs == 2
    # WHEN aggregating data on failed jobs
    jobs_data = sample_store.aggregate_failed()
    # THEN it should return a list of dicts per job type with count
    assert isinstance(jobs_data, list)
    # ... it should exclude "cancelled" jobs
    assert len(jobs_data) == 1
    assert jobs_data[0]['name'] == 'samtools_mpileup'
    assert jobs_data[0]['count'] == 1



# -*- coding: utf-8 -*-
import datetime

import pytest

from trailblazer.mip import sacct


@pytest.mark.parametrize('time_str, expected_seconds', [
    ('02:53:08', (2 * 60 * 60) + (53 * 60)),
    ('3-07:13:56', (3 * 24 * 60 * 60) + (7 * 60 * 60) + (13 * 60)),
    ('18:47.020', (18 * 60)),
    ('00:00:35', 0)
])
def test_time_to_sec(time_str, expected_seconds):
    # GIVEN a time string
    # WHEN parsing it into number of seconds
    seconds = sacct.time_to_sec(time_str)
    # THEN it should match expected number of seconds (skipping individual secs)
    assert seconds == expected_seconds


@pytest.mark.parametrize('row, expected_job', [
    (['813887', 'gatk_genotypegvcfs_case_BOTH', 'prod001', 'core', '16', '10:03.096',
      '00:02:01', '2017-06-19T00:44:46', '2017-06-19T00:46:47', 'COMPLETED', '0:0'],
     dict(step='gatk_genotypegvcfs', context='case', is_completed=True)),
    (['815038', 'fastqc_ACC2698A3', 'prod001', 'core', '8', '00:00:00', '00:37:24',
      '2017-06-20T12:57:45', 'Unknown', 'RUNNING', '0:0'],
     dict(step='fastqc', context='ACC2698A3', is_completed=False)),
])
def test_convert_job(row, expected_job):
    # GIVEN a sacct row
    # WHEN parsing it for extracted data
    job = sacct.convert_job(row)
    # THEN it should have an integer for a job id
    assert isinstance(job['id'], int)
    # ... the step should be the job type
    assert job['step'] == expected_job['step']
    # ... the context should be the family/sample id
    assert job['context'] == expected_job['context']
    # ... it should be marked as completed only if that is the state
    assert job['is_completed'] == expected_job['is_completed']
    # ... the "start" should be a date
    assert isinstance(job['start'], datetime.datetime)
    # ... the "end" should be a date unless the job is "running"
    if job['state'] == 'RUNNING':
        assert job['end'] is None
    else:
        assert isinstance(job['end'], datetime.datetime)


def test_parse_sacct(files_raw):
    # GIVEN a sacct file with a number of row (1 header, rest jobs)
    lines = len(files_raw['sacct'])
    # WHEN parsing the sacct file
    jobs = sacct.parse_sacct(files_raw['sacct'])
    # THEN it should find all relevant rows
    assert len(jobs) == (lines - 1)


def test_filter_jobs(failed_sacct_jobs):
    # GIVEN a sacct output with COMPLETED, FAILED, and CANCELLED jobs
    completed_count = 17
    failed_count = 2
    cancelled_count = 34
    # WHEN filtering on "failed" jobs
    failed_jobs = sacct.filter_jobs(failed_sacct_jobs, failed=True)
    # THEN it should return jobs which are "failed" or "cancelled"
    assert len(failed_jobs) == (failed_count + cancelled_count)

    # WHEN filtering on "OK" jobs
    ok_jobs = sacct.filter_jobs(failed_sacct_jobs, failed=False)
    # THEN it should return jobs which are "completed"
    assert len(ok_jobs) == completed_count



# -*- coding: utf-8 -*-
"""Test MIP files"""

from trailblazer.mip import files


def test_parse_config(files_raw) -> dict:
    """
    Args:
    files_raw (dict): With dicts from files
    """

    # GIVEN config data of a "sharp" run (not dry run)
    config_raw = files_raw['config']

    # WHEN parsing the MIP output config
    config_data = files.parse_config(config_raw)

    # THEN it should work
    assert isinstance(config_data, dict)

    # THEN it should work and the dry run check should be "yes"
    assert config_data['is_dryrun'] is True

    # ... and should include all samples
    assert len(config_raw['analysis_type']) == len(config_data['samples'])

    # Build dict for return data
    config_test_data = {
        'config_path': '/path_to/cases/case/analysis/case_config.yaml',
        'email': 'test.person@test.se',
        'case': 'case',
        'log_path': 'tests/fixtures/case/mip_2019-07-04T10:47:15.log',
        'sampleinfo_path': '/path_to/cases/case/analysis/case_qc_sample_info.yaml',
        'out_dir': '/path_to/cases/case/analysis',
        'priority': 'normal',
        }

    # Check returns from def
    for key, value in config_test_data.items():
        assert config_data[key] == value


def test_parse_sampleinfo(files_raw):
    """
    Args:
    files_raw (dict): With dicts from files
    """

    # GIVEN sampleinfo input from a finished analysis
    sampleinfo_raw = files_raw['sampleinfo']

    # WHEN parsing the MIP output sampleinfo
    sampleinfo_data = files.parse_sampleinfo(sampleinfo_raw)

    # THEN it should work
    assert isinstance(sampleinfo_data, dict)

    # THEN it should mark the run as "finished"
    assert sampleinfo_data['is_finished'] is True

    # More in-depth testing
    # Family data
    # Build dict for family return data
    sampleinfo_test_data = {
        'case': 'case',
        'genome_build': 'grch37',
        'pedigree_path': '/path_to/case/case.fam',
        'qcmetrics_path': '/path_to/case/case_qc_metrics.yaml',
        'sv_combinevariantcallsets_path':
            '/path_to/case/case/sv_combinevariantcallsets/case_comb.vcf',
        'version': 'v7.1.0',
        }

    # Check returns from def 1
    for key, value in sampleinfo_test_data.items():
        assert sampleinfo_data[key] == value

    # Sample data
    # Build dict for sample return data
    sampleinfo_test_sample_data = {
        'bam': '/path_to/case/mother/gatk_baserecalibration/mother_lanes_1_sorted_md_brecal.bam',
        'chanjo_sexcheck': (
            '/path_to/case/mother/chanjo_sexcheck/mother_lanes_1_sorted_md_brecal_sex.tsv'),
        'id': 'mother',
        'sambamba': ('/path_to/case/mother/sambamba_depth'
                     '/mother_lanes_1_sorted_md_brecal_coverage.bed'),
        'sex': 'female',
        'subsample_mt': ('/path_to/case/mother/samtools_subsample_mt'
                         '/mother_lanes_1_sorted_md_brecal_subsample_MT.bam'),
        'vcf2cytosure': '/path_to/case/case/vcf2cytosure_ar/case_cyto.mother.cgh',
        }

    # Check returns from def 2
    for key, value in sampleinfo_test_sample_data.items():
        for sample_data in sampleinfo_data['samples']:

            if sample_data['id'] != sampleinfo_test_sample_data['id']:
                continue

            assert sample_data[key] == value

    assert len(sampleinfo_raw['analysis_type']) == len(sampleinfo_data['samples'])

    # Snv data
    # Build dict for snv return data
    sampleinfo_test_snv_data = {
        'clinical_vcf': ('/path_to/case/case/endvariantannotationblock'
                         '/case_gatkcomb_rhocall_vt_frqf_cadd_vep_parsed_snpeff_ranked.selected'
                         '.vcf.gz'),
        'research_vcf': ('/path_to/case/case/endvariantannotationblock'
                         '/case_gatkcomb_rhocall_vt_frqf_cadd_vep_parsed_snpeff_ranked.vcf.gz'),
        'bcf': '/path_to/case/case/gatk_combinevariantcallsets/case_gatkcomb.bcf',
        }

    # Check returns from def 3
    for key, value in sampleinfo_test_snv_data.items():
        assert sampleinfo_data['snv'][key] == value

    # SV data
    # Build dict for sv return data
    sampleinfo_test_sv_data = {
        'clinical_vcf': ('/path_to/case/case/sv_reformat/case_comb_ann_vep_parsed_ranked.selected'
                         '.vcf.gz'),
        'research_vcf': '/path_to/case/case/sv_reformat/case_comb_ann_vep_parsed_ranked.vcf.gz',
        'bcf': '/path_to/case/case/sv_combinevariantcallsets/case_comb.bcf',
        'merged': '/path_to/case/case/sv_combinevariantcallsets/case_comb.vcf',
        }

    # Check returns from def 4
    for key, value in sampleinfo_test_sv_data.items():
        assert sampleinfo_data['sv'][key] == value

    # Peddy data
    # Build dict for peddy return data
    sampleinfo_test_peddy_data = {
        'ped_check': '/path_to/case/case/peddy_ar/case_gatkcomb.ped_check.csv',
        'ped': '/path_to/case/case/peddy_ar/case_gatkcomb.peddy.ped',
        'sex_check': '/path_to/case/case/peddy_ar/case_gatkcomb.sex_check.csv',
        }

    # Check returns from def 5
    for key, value in sampleinfo_test_peddy_data.items():
        assert sampleinfo_data['peddy'][key] == value


def test_parse_qcmetrics(files_raw):
    """
    Args:
    files_raw (dict): With dicts from files
    """

    # GIVEN qc metrics input from an analysis
    qcmetrics_raw = files_raw['qcmetrics']

    # WHEN parsing it
    qcmetrics_data = files.parse_qcmetrics(qcmetrics_raw)

    # THEN it should work ;-)
    assert isinstance(qcmetrics_data, dict)

    # Version data
    # Build dict for version return data
    qcmetrics_test_version_data = {
        'gatk': '3.8',
        'manta': '1.5.0',
        'vep': 'v95',
        }

    # Check returns from def
    for key, value in qcmetrics_test_version_data.items():
        assert qcmetrics_data['versions'][key] == value

    # Sample data
    # Build dict for sample return data
    qcmetrics_test_sample_data = {
        'at_dropout': '1.716704',
        'duplicates': 0.0379523291229131,
        'id': 'mother',
        'insert_size_standard_deviation': 94.353778,
        'gc_dropout': '0.214813',
        'mapped': 0.9974176575073073,
        'median_insert_size': '409',
        'plink_sex': 'female',
        'predicted_sex': 'female',
        'reads': 600006004,
        'strand_balance': 0.50162,
        'target_coverage': 28.643247,
        }

    # Check returns from def
    for key, value in qcmetrics_test_sample_data.items():
        for sample_data in qcmetrics_data['samples']:

            if sample_data['id'] != qcmetrics_test_sample_data['id']:
                continue

            assert sample_data[key] == value

    # Sample coverage data
    # Build dict for sample coverage return data
    qcmetrics_test_sample_cov_data = {
        10: '0.98974',
        20: '0.935455',
        50: '0.002685',
        100: '0.000101',
        }

    # Check returns from def
    for key, value in qcmetrics_test_sample_cov_data.items():
        for sample_data in qcmetrics_data['samples']:

            if sample_data['id'] != qcmetrics_test_sample_data['id']:
                continue

            assert sample_data['completeness_target'][key] == value



# -*- coding: utf-8 -*-
import pytest

from trailblazer.mip import sacct
from trailblazer.mip import start


@pytest.fixture(scope='session')
def failed_sacct_jobs():
    with open('tests/fixtures/sacct/failed.log.status') as stream:
        sacct_jobs = sacct.parse_sacct(stream)
    return sacct_jobs


@pytest.fixture(scope='session')
def mip_cli():
    _mip_cli = start.MipCli(script='test/fake_mip.pl', pipeline='rd_dna')
    return _mip_cli



from pathlib import Path
import shutil

import click


@click.command()
@click.option('-f', '--force', is_flag=True, help='skip sanity checks')
@click.option('-y', '--yes', is_flag=True, help='skip manual confirmations')
@click.argument('analysis_id', type=int)
@click.pass_context
def delete(context, force, yes, analysis_id):
    """Mark analysis as deleted in db and remove analysis folder on disk."""
    analysis_obj = context.obj['store'].analysis(analysis_id)
    if analysis_obj is None:
        print(click.style('analysis log not found', fg='red'))
        context.abort()

    print(click.style(f"{analysis_obj.family}: {analysis_obj.status}"))

    if analysis_obj.is_temp:
        if yes or click.confirm(f"remove analysis log?"):
            analysis_obj.delete()
            context.obj['store'].commit()
            print(click.style(f"analysis deleted: {analysis_obj.family}", fg='blue'))
    else:
        if analysis_obj.is_deleted:
            print(click.style(f"{analysis_obj.family}: already deleted", fg='red'))
            context.abort()

        if Path(analysis_obj.out_dir).exists() or force:
            root_dir = context.obj['store'].families_dir
            family_dir = analysis_obj.out_dir
            if not force and (len(family_dir) <= len(root_dir) or root_dir not in family_dir):
                print(click.style(f"unknown analysis output dir: {analysis_obj.out_dir}", fg='red'))
                print(click.style("use '--force' to override"))
                context.abort()

            if yes or click.confirm(f"remove analysis output: {analysis_obj.out_dir}?"):
                shutil.rmtree(analysis_obj.out_dir, ignore_errors=True)
                analysis_obj.is_deleted = True
                context.obj['store'].commit()
                print(click.style(f"analysis deleted: {analysis_obj.family}", fg='blue'))
        else:
            print(click.style(f"analysis output doesn't exist: {analysis_obj.out_dir}", fg='red'))
            print(click.style("use '--force' to override"))
            context.abort()



# -*- coding: utf-8 -*-
import trailblazer


def test_base(invoke_cli):
    # GIVEN the cli
    # WHEN asking for the version
    result = invoke_cli(['--version'])
    # THEN it should print the name and version of the tool only
    assert trailblazer.__title__ in result.output
    assert trailblazer.__version__ in result.output



# -*- coding: utf-8 -*-


def test_build_command(mip_cli):
    # GIVEN just a config
    mip_config = 'tests/fixtures/global_config.yaml'
    case_id = 'angrybird'
    # WHEN building the MIP command
    mip_command = mip_cli.build_command(case=case_id, config=mip_config)
    # THEN it should use the correct options
    assert mip_command == [mip_cli.script, mip_cli.pipeline, case_id, '--config_file', mip_config]



# -*- coding: utf-8 -*-
from flask_alchy import Alchy

from trailblazer.store import BaseHandler, models


class TrailblazerAlchy(Alchy, BaseHandler):
    pass


store = TrailblazerAlchy(Model=models.Model)



# -*- coding: utf-8 -*-
import logging
from pathlib import Path
import subprocess

import click
import coloredlogs
import ruamel.yaml

import trailblazer
from trailblazer.store import Store
from trailblazer.log import LogAnalysis
from trailblazer.mip.start import MipCli
from trailblazer.mip.files import parse_config
from trailblazer.mip.miplog import job_ids
from trailblazer.exc import MissingFileError, MipStartError
from .utils import environ_email
from .clean import clean
from .delete import delete
from .ls import ls_cmd
from .check import check

LOG = logging.getLogger(__name__)


@click.group()
@click.option('-c', '--config', type=click.File())
@click.option('-d', '--database', help='path/URI of the SQL database')
@click.option('-r', '--root', help='families root directory')
@click.option('-l', '--log-level', default='INFO')
@click.version_option(trailblazer.__version__, prog_name=trailblazer.__title__)
@click.pass_context
def base(context, config, database, root, log_level):
    """Trailblazer - Simplify running MIP!"""
    coloredlogs.install(level=log_level)

    context.obj = ruamel.yaml.safe_load(config) if config else {}
    context.obj['database'] = database or context.obj.get('database')
    context.obj['root'] = root or context.obj.get('root')
    context.obj['store'] = Store(context.obj['database'], context.obj['root'])


@base.command('log')
@click.option('-s', '--sampleinfo', type=click.Path(exists=True), help='sample info file')
@click.option('-a', '--sacct', type=click.Path(exists=True), help='sacct job info file')
@click.option('-q', '--quiet', is_flag=True, help='supress outputs')
@click.argument('config', type=click.File())
@click.pass_context
def log_cmd(context, sampleinfo, sacct, quiet, config):
    """Log an analysis.

    CONFIG: MIP config file for an analysis
    """
    log_analysis = LogAnalysis(context.obj['store'])
    try:
        new_run = log_analysis(config, sampleinfo=sampleinfo, sacct=sacct)
    except MissingFileError as error:
        click.echo(click.style(f"Skipping, missing Sacct file: {error.message}", fg='red'))
        return
    except KeyError as error:
        print(click.style(f"unexpected output, missing key: {error.args[0]} in {config}", fg='red'))
        return
    if new_run is None:
        if not quiet:
            click.echo(click.style('Analysis already logged', fg='yellow'))
    else:
        message = f"New log added: {new_run.family} ({new_run.id}) - {new_run.status}"
        click.echo(click.style(message, fg='green'))


@base.command()
@click.option('-c', '--mip-config', type=click.Path(exists=True), help='MIP config')
@click.option('-e', '--email', help='email for logging user')
@click.option('-p', '--priority', type=click.Choice(['low', 'normal', 'high']), default='normal')
@click.option('-d', '--dryrun', is_flag=True, help='only generate SBATCH scripts')
@click.option('--command', is_flag=True, help='only show the MIP command')
@click.option('-sw', '--start-with', help='start the pipeline beginning with program,\
                                           see format for program in mip.pl')
@click.argument('case', required=False)
@click.pass_context
def start(context, mip_config, email, priority, dryrun, command, start_with, case):
    """Start a new analysis."""
    mip_cli = MipCli(context.obj['script'], context.obj['pipeline'])
    mip_config = mip_config or context.obj['mip_config']
    email = email or environ_email()
    kwargs = dict(config=mip_config, case=case, priority=priority,
                  email=email, dryrun=dryrun, start_with=start_with)
    if command:
        mip_command = mip_cli.build_command(**kwargs)
        click.echo(' '.join(mip_command))
    else:
        try:
            mip_cli(**kwargs)
            if not dryrun:
                context.obj['store'].add_pending(case, email=email)
        except MipStartError as error:
            click.echo(click.style(error.message, fg='red'))


@base.command()
@click.option('--reset', is_flag=True, help='reset database before setting up tables')
@click.option('--force', is_flag=True, help='bypass manual confirmations')
@click.pass_context
def init(context, reset, force):
    """Setup the database."""
    existing_tables = context.obj['store'].engine.table_names()
    if force or reset:
        if existing_tables and not force:
            message = f"Delete existing tables? [{', '.join(existing_tables)}]"
            click.confirm(click.style(message, fg='yellow'), abort=True)
        context.obj['store'].drop_all()
    elif existing_tables:
        click.echo(click.style("Database already exists, use '--reset'", fg='red'))
        context.abort()

    context.obj['store'].setup()
    message = f"Success! New tables: {', '.join(context.obj['store'].engine.table_names())}"
    click.echo(click.style(message, fg='green'))


@base.command()
@click.argument('root_dir', type=click.Path(exists=True), required=False)
@click.pass_context
def scan(context, root_dir):
    """Scan a directory for analyses."""
    root_dir = root_dir or context.obj['root']
    config_files = Path(root_dir).glob('*/analysis/*_config.yaml')
    for config_file in config_files:
        LOG.debug("found analysis config: %s", config_file)
        with config_file.open() as stream:
            context.invoke(log_cmd, config=stream, quiet=True)

    context.obj['store'].track_update()


@base.command()
@click.option('--name', help='Name of new user to add')
@click.argument('email')
@click.pass_context
def user(context, name, email):
    """Add a new or display information about an existing user."""
    existing_user = context.obj['store'].user(email)
    if existing_user:
        click.echo(existing_user.to_dict())
    elif name:
        new_user = context.obj['store'].add_user(name, email)
        click.echo(click.style(f"New user added: {email} ({new_user.id})", fg='green'))
    else:
        click.echo(click.style('User not found', fg='yellow'))


@base.command()
@click.option('-j', '--jobs', is_flag=True, help='only print job ids')
@click.argument('analysis_id', type=int)
@click.pass_context
def cancel(context, jobs, analysis_id):
    """Cancel all jobs in a run."""
    analysis_obj = context.obj['store'].analysis(analysis_id)
    if analysis_obj is None:
        click.echo('analysis not found')
        context.abort()
    elif analysis_obj.status != 'running':
        click.echo(f"analysis not running: {analysis_obj.status}")
        context.abort()

    config_path = Path(analysis_obj.config_path)
    with config_path.open() as config_stream:
        config_raw = ruamel.yaml.safe_load(config_stream)
    config_data = parse_config(config_raw)

    log_path = Path(f"{config_data['log_path']}")
    if not log_path.exists():
        click.echo(f"missing MIP log file: {log_path}")
        context.abort()

    with log_path.open() as log_stream:
        all_jobs = job_ids(log_stream)

    if jobs:
        for job_id in all_jobs:
            click.echo(job_id)
    else:
        for job_id in all_jobs:
            LOG.debug(f"cancelling job: {job_id}")
            process = subprocess.Popen(['scancel', job_id])
            process.wait()

        analysis_obj.status = 'canceled'
        context.obj['store'].commit()
        click.echo('cancelled analysis successfully!')


base.add_command(delete)
base.add_command(ls_cmd)
base.add_command(clean)
base.add_command(check)



# -*- coding: utf-8 -*-
"""Test MIP files"""

from trailblazer.mip import trending


def test_parse_mip_analysis(files_raw) -> dict:
    """
    Args:
    files_raw (dict): With dicts from files
    """
    # GIVEN config data of a "sharp" run (not dry run)
    mip_config_raw = files_raw['config']

    # GIVEN sampleinfo input from a finished analysis
    sampleinfo_raw = files_raw['sampleinfo']

    # GIVEN qc metrics input from an analysis
    qcmetrics_raw = files_raw['qcmetrics']

    trend_data = trending.parse_mip_analysis(mip_config_raw, qcmetrics_raw, sampleinfo_raw)

    # Build dict for family return data
    trend_test_data = {
        'case': 'case',
        'genome_build': 'grch37',
        'mip_version': 'v7.1.0',
        'rank_model_version': '1.25'
    }

    # Check returns from def
    for key, value in trend_test_data.items():
        assert trend_data[key] == value

    # Check sample id
    assert 'mother' in trend_data['sample_ids']

    # Build dict for sample return data
    trend_test_sample_data = {
        'at_dropout': {
            'mother': '1.716704',
            },
        'analysis_sex': {
            'mother': 'female',
            },
        'duplicates': {
            'mother': 3.7952329122913104,
            },
        'insert_size_standard_deviation': {
            'mother': 94.353778,
            },
        'mapped_reads': {
            'mother': 99.74176575073072,
            },
        'median_insert_size': {
            'mother': '409',
            },
        'gc_dropout': {
            'mother': '0.214813',
            },
    }

    # Check returns from def
    for metric in trend_test_sample_data:
        for sample, value in trend_test_sample_data[metric].items():
            assert trend_data[metric][sample] == value
